'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCompilerState = getCompilerState;
exports.getBabel = getBabel;
exports.getOptions = getOptions;
exports.initializeCompilerState = initializeCompilerState;
exports.postprocess = postprocess;

var _babelTypesPatch = require('./util/babelTypesPatch');

var _babelTypesPatch2 = _interopRequireDefault(_babelTypesPatch);

var _lscNodeTypes = require('./lscNodeTypes');

var _stdlib = require('./stdlib');

var _runtime = require('./runtime');

var _imports = require('./imports');

var _inlinedOperators = require('./inlinedOperators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Singleton object, unique per Program traversal, holding relevant state.
var compilerState = {
  // DI Babel instance. Avoid unless necessary; prefer types.lsc
  babel: null

  // Root program path
  , programPath: null

  // Babel Plugin options
  , opts: null

  //// Preambles
  // stdlib state
  , stdlib: null
  // helpers state
  , helpers: null
  // lightscript-runtime state
  , runtime: null,

  inlinedOperatorsEnabled: true
};

function getCompilerState() {
  return compilerState;
}

function getBabel() {
  return compilerState.babel;
}

function getOptions() {
  return compilerState.opts;
}

function initializeCompilerState(initialState) {
  compilerState = initialState;
  var _compilerState = compilerState,
      babel = _compilerState.babel,
      opts = _compilerState.opts;

  (0, _babelTypesPatch2.default)(babel.types);
  (0, _lscNodeTypes.registerLightscriptNodeTypes)(babel.types);

  compilerState.stdlib = (0, _stdlib.initializeStdlib)(opts);
  compilerState.inlinedOperatorsEnabled = (0, _inlinedOperators.getInlinedOperatorsEnabled)(opts);
  compilerState.runtime = (0, _runtime.initializeRuntime)(opts);

  return compilerState;
}

function postprocess(state) {
  var stdlib = state.stdlib,
      programPath = state.programPath,
      runtime = state.runtime;

  (0, _imports.insertImports)(stdlib.imports, programPath, stdlib.useRequire || state.opts.useRequire);
  return (0, _imports.insertImports)(runtime.imports, programPath, state.opts.useRequire);
}