'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformForInArrayStatement = transformForInArrayStatement;
exports.transformForInObjectStatement = transformForInObjectStatement;

var _lightscriptTypes = require('./lightscriptTypes');

var _lightscriptTypes2 = _interopRequireDefault(_lightscriptTypes);

var _blocks = require('./blocks');

var _is = require('./is');

var _is2 = _interopRequireDefault(_is);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function generateForInIterator(path, type) {
  var idx = path.node.idx || path.scope.generateUidIdentifier("i");

  // _i = 0
  var initDeclarations = [_lightscriptTypes2.default.variableDeclarator(idx, _lightscriptTypes2.default.numericLiteral(0))];

  // (_obj|_arr) = <expr>
  var refId = (0, _is2.default)("Identifier", path.node[type]) ? path.node[type] : function () {
    // if the target of iteration is a complex expression,
    // create a reference so it only evaluates once
    var refName = type === "object" ? "obj" : "arr";
    var refId = path.scope.generateUidIdentifier(refName);
    initDeclarations.unshift(_lightscriptTypes2.default.variableDeclarator(refId, path.node[type]));
    return refId;
  }();

  // _keys = Object.keys(_obj)
  var keys = void 0;
  if (type === "object") {
    keys = path.scope.generateUidIdentifier("keys");
    initDeclarations.push(_lightscriptTypes2.default.variableDeclarator(keys, _lightscriptTypes2.default.callExpression(_lightscriptTypes2.default.memberExpression(_lightscriptTypes2.default.identifier("Object"), _lightscriptTypes2.default.identifier("keys")), [refId])));
  }

  // _len = (_keys | _arr).length
  var len = path.scope.generateUidIdentifier("len");
  initDeclarations.push(_lightscriptTypes2.default.variableDeclarator(len, _lightscriptTypes2.default.memberExpression(type === "object" ? keys : refId, _lightscriptTypes2.default.identifier("length"))));

  var init = _lightscriptTypes2.default.variableDeclaration("let", initDeclarations);
  // _i < _len
  var test = _lightscriptTypes2.default.binaryExpression("<", idx, len);
  // _i++
  var update = _lightscriptTypes2.default.updateExpression("++", idx);

  (0, _blocks.ensureBlockBody)(path);
  var innerDeclarations = [];
  if (type === "object") {
    // _k = _keys[_i]
    var key = path.node.key || path.scope.generateUidIdentifier("k");
    innerDeclarations.push(_lightscriptTypes2.default.variableDeclaration("const", [_lightscriptTypes2.default.variableDeclarator(key, _lightscriptTypes2.default.memberExpression(keys, idx, true))]));

    // val = _obj[_k]
    if (path.node.val) {
      innerDeclarations.push(_lightscriptTypes2.default.variableDeclaration("const", [_lightscriptTypes2.default.variableDeclarator(path.node.val, _lightscriptTypes2.default.memberExpression(refId, key, true))]));
    }
  } else {
    // elem = _arr[_i]
    if (path.node.elem) {
      innerDeclarations.push(_lightscriptTypes2.default.variableDeclaration("const", [_lightscriptTypes2.default.variableDeclarator(path.node.elem, _lightscriptTypes2.default.memberExpression(refId, idx, true))]));
    }
  }

  if (innerDeclarations.length > 0) {
    path.get("body").unshiftContainer("body", innerDeclarations);
  }

  return _lightscriptTypes2.default.forStatement(init, test, update, path.node.body);
}

function transformForInArrayStatement(path) {
  return path.replaceWith(generateForInIterator(path, "array"));
}

function transformForInObjectStatement(path) {
  return path.replaceWith(generateForInIterator(path, "object"));
}