'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformMatchExpression = transformMatchExpression;
exports.transformMatchStatement = transformMatchStatement;

var _lightscriptTypes = require('./lightscriptTypes');

var _lightscriptTypes2 = _interopRequireDefault(_lightscriptTypes);

var _blocks = require('./blocks');

var _is = require('./is');

var _ref5 = require('./ref');

var _destructure = require('./destructure');

var _logical = require('./logical');

var _helpers = require('./helpers');

var _runtime = require('./runtime');

var _getLoc = require('ast-loc-utils/lib/getLoc');

var _getLoc2 = _interopRequireDefault(_getLoc);

var _placeAtLoc = require('ast-loc-utils/lib/placeAtLoc');

var _placeAtLoc2 = _interopRequireDefault(_placeAtLoc);

var _placeAtNode = require('ast-loc-utils/lib/placeAtNode');

var _placeAtNode2 = _interopRequireDefault(_placeAtNode);

var _getSurroundingLoc = require('ast-loc-utils/lib/getSurroundingLoc');

var _getSurroundingLoc2 = _interopRequireDefault(_getSurroundingLoc);

var _span = require('ast-loc-utils/lib/span');

var _span2 = _interopRequireDefault(_span);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

// TODO: factor this out to ast-loc-utils
function _traverse(src, visit) {
  visit(src);
  for (var key in src) {
    if (key[0] === "_") continue;
    var val = src[key];
    if (val) {
      if (val.type) {
        _traverse(val, visit);
      } else if (Array.isArray(val)) {
        for (var _i3 = 0, _len3 = val.length; _i3 < _len3; _i3++) {
          var _e = val[_i3];

          _traverse(_e, visit);
        }
      }
    }
  }return src;
}

function isUndefined(node) {
  return node.type === "Identifier" && node.name === "undefined";
}

function isSignedNumber(node) {
  return node.type === "UnaryExpression" && (node.operator === "+" || node.operator === "-") && node.argument.type === "NumericLiteral";
}

function transformGuardId(idPath, bindingAccessors) {
  var node = idPath.node;

  if (node._guardWasTransformed) return; // Prevent infinite recursion

  if (bindingAccessors[node.name]) {
    var loc = (0, _getLoc2.default)(node);
    var replacement = _lightscriptTypes2.default.cloneDeep(bindingAccessors[node.name]);
    _traverse(replacement, function (n) {
      // Prevent infinite recursion
      if (n.type === "Identifier") n._guardWasTransformed = true;
      return (0, _placeAtLoc2.default)(n, loc);
    });
    return idPath.replaceWith(replacement);
  }
}

function transformInnerGuard(mtch, path, bindingAccessors) {
  path.traverse({
    ReferencedIdentifier(idPath) {
      transformGuardId(idPath, bindingAccessors);
    } });
  return path.node;
}

function transformPredicate(mtch, path) {
  var discriminantRef = mtch.discriminantRef;

  path.traverse({
    MatchPlaceholderExpression(phPath) {
      return phPath.replaceWith(discriminantRef);
    } });
  return path.node;
}

function isStrictEqualityLiteral(node) {
  return isSignedNumber(node) || isUndefined(node) || (0, _is.isa)(node, "Literal") && !(0, _is.isa)(node, "RegExpLiteral");
}

function isPredicate(node) {
  return (0, _is.isa)(node, "CallExpression");
}

function transformAtom(mtch, path) {
  var discriminantRef = mtch.discriminantRef;
  var node = path.node;

  if ((0, _is.isa)(node, "LogicalExpression")) {
    node.left = transformAtom(mtch, path.get("left"));
    node.right = transformAtom(mtch, path.get("right"));
    return node;
  } else if ((0, _is.isa)(node, "UnaryExpression") && node.operator === "!") {
    node.argument = transformAtom(mtch, path.get("argument"));
    return node;
  } else if (isStrictEqualityLiteral(node)) {
    return (0, _placeAtNode2.default)(_lightscriptTypes2.default.binaryExpression("===", discriminantRef, node), node);
  } else if (isPredicate(node)) {
    return transformPredicate(mtch, path);
  } else {
    return (0, _placeAtNode2.default)(_lightscriptTypes2.default.callExpression((0, _placeAtNode2.default)(_lightscriptTypes2.default.clone((0, _runtime.runtime)('isMatch')), node), [node, discriminantRef]), node);
  }
}

function transformAtoms(mtch, atomPaths) {
  return (0, _logical.createOrExpression)(function () {
    var _arr = [];

    for (var _i4 = 0, _len4 = atomPaths.length; _i4 < _len4; _i4++) {
      var _atomPath = atomPaths[_i4];

      _arr.push(transformAtom(mtch, _atomPath));
    }

    return _arr;
  }());
}

function transformMatchTest(mtch, casePath) {
  var _casePath$node = casePath.node,
      outerGuard = _casePath$node.outerGuard,
      atoms = _casePath$node.atoms,
      binding = _casePath$node.binding,
      assertive = _casePath$node.assertive,
      innerGuard = _casePath$node.innerGuard;
  var discriminantRef = mtch.discriminantRef;

  var conditions = [];

  if (outerGuard) {
    conditions.push(outerGuard);
  }if (atoms && atoms.length > 0) {
    conditions.push(transformAtoms(mtch, casePath.get("atoms")));
  }if (binding && assertive) {
    conditions.push((0, _destructure.matchesDestructuringPattern)(discriminantRef, binding));
  }if (innerGuard) {
    if (binding) {
      // Transform referenced identifiers in the inner guard into accessors
      // into the matched pattern.
      var bindingAccessors = {};
      (0, _destructure.accessors)(binding, discriminantRef, bindingAccessors);
      var nextInnerGuard = transformInnerGuard(mtch, casePath.get("innerGuard"), bindingAccessors);
      conditions.push(nextInnerGuard);
    } else {
      conditions.push(innerGuard);
    }
  }return (0, _logical.createAndExpression)(conditions);
}

function transformBoundConsequent(_ref2, binding, consequent) {
  var discriminantRef = _ref2.discriminantRef;

  var block = (0, _blocks.toBlockStatement)(consequent);
  block.body.unshift((0, _placeAtNode2.default)(_lightscriptTypes2.default.variableDeclaration("const", [(0, _placeAtNode2.default)(_lightscriptTypes2.default.variableDeclarator(binding, discriminantRef), binding)]), binding));
  return block;
}

function transformMatchConsequent(mtch, casePath) {
  var _casePath$node2 = casePath.node,
      binding = _casePath$node2.binding,
      consequent = _casePath$node2.consequent;

  if (binding) {
    return transformBoundConsequent(mtch, binding, consequent);
  } else {
    return (0, _blocks.toBlockStatement)(consequent);
  }
}

function transformMatchCase(mtch, casePath) {
  var _ref;

  var c = casePath.node;
  return {
    test: ((_ref = c.outerGuard) == null ? null : _ref.type) !== "MatchElse" ? transformMatchTest(mtch, casePath) : null,
    consequent: transformMatchConsequent(mtch, casePath)
  };
}

function transformMatchCases(mtch, _ref3) {
  var _ref4 = _toArray(_ref3),
      casePath = _ref4[0],
      rest = _ref4.slice(1);

  var c = casePath.node;

  var _transformMatchCase = transformMatchCase(mtch, casePath),
      test = _transformMatchCase.test,
      consequent = _transformMatchCase.consequent;

  if (rest.length > 0) {
    var loc = (0, _getSurroundingLoc2.default)([c, mtch.lastCase]);
    return (0, _placeAtLoc2.default)(_lightscriptTypes2.default.ifStatement(test, consequent, transformMatchCases(mtch, rest)), loc);
  } else if (test) {
    return (0, _placeAtNode2.default)(_lightscriptTypes2.default.ifStatement(test, consequent), c);
  } else {
    return consequent;
  }
} // Computed values needed in recursive descent.


function getMatchInfo(path, discriminantRef) {
  var node = path.node;

  return {
    path, node, discriminantRef
    // Track last case for source map positioning
    , lastCase: node.cases[node.cases.length - 1]
  };
}

function transformMatchExpression(path) {
  var node = path.node;


  var argRef = (0, _placeAtLoc2.default)(path.scope.generateUidIdentifier("it"), (0, _span2.default)((0, _getLoc2.default)(node), 1));
  var matchBody = transformMatchCases(getMatchInfo(path, argRef), path.get("cases"));

  var iife = (0, _placeAtNode2.default)(_lightscriptTypes2.default.callExpression((0, _placeAtNode2.default)(_lightscriptTypes2.default.arrowFunctionExpression([argRef], (0, _placeAtNode2.default)(_lightscriptTypes2.default.blockStatement([matchBody]), node)), node), [node.discriminant]), node);

  return path.replaceWith(iife);
}

function transformMatchStatement(path) {
  var discriminant = path.node.discriminant;

  var _undeclaredRef = (0, _ref5.undeclaredRef)(path, discriminant, "it"),
      ref = _undeclaredRef.ref,
      declarator = _undeclaredRef.declarator,
      isComplex = _undeclaredRef.isComplex;

  if (isComplex) {
    path.insertBefore((0, _placeAtNode2.default)(_lightscriptTypes2.default.variableDeclaration("const", [declarator]), declarator));
  }return path.replaceWith(transformMatchCases(getMatchInfo(path, ref), path.get("cases")));
}